@@ -42,13 +42,27 @@
 #include "msvc_nmake.h"
 #include "option.h"
 #include "cesdkhandler.h"
+
 #include <qregexp.h>
 #include <qhash.h>
 #include <qdir.h>
+
+#include <windows/registry_p.h>
+
 #include <time.h>
 
 QT_BEGIN_NAMESPACE
 
+static QString nmakePathList(const QStringList &list)
+{
+    QStringList pathList;
+    foreach (const QString &path, list)
+        pathList.append(QDir::cleanPath(path));
+
+    return QDir::toNativeSeparators(pathList.join(QLatin1Char(';')))
+            .replace('#', QStringLiteral("^#")).replace('$', QStringLiteral("$$"));
+}
+
 NmakeMakefileGenerator::NmakeMakefileGenerator() : Win32MakefileGenerator(), init_flag(false), usePCH(false)
 {
 
@@ -58,17 +72,8 @@ bool
 NmakeMakefileGenerator::writeMakefile(QTextStream &t)
 {
     writeHeader(t);
-    if(!project->values("QMAKE_FAILED_REQUIREMENTS").isEmpty()) {
-        const ProStringList &qut = project->values("QMAKE_EXTRA_TARGETS");
-        for (ProStringList::ConstIterator it = qut.begin(); it != qut.end(); ++it)
-            t << *it << " ";
-        t << "all first clean:" << "\n\t"
-          << "@echo \"Some of the required modules ("
-          << var("QMAKE_FAILED_REQUIREMENTS") << ") are not available.\"" << "\n\t"
-          << "@echo \"Skipped.\"" << endl << endl;
-        writeMakeQmake(t);
+    if (writeDummyMakefile(t))
         return true;
-    }
 
     if(project->first("TEMPLATE") == "app" ||
        project->first("TEMPLATE") == "lib" ||
@@ -108,6 +113,86 @@ NmakeMakefileGenerator::writeMakefile(QTextStream &t)
                             qPrintable(variables["QMAKESPEC"].first().toQString()));
                     return false;
                 }
+            } else if (project->isActiveConfig(QStringLiteral("winrt"))) {
+                QString arch = project->first("VCPROJ_ARCH").toQString().toLower();
+                QString compiler;
+                QString compilerArch;
+                if (arch == QStringLiteral("arm")) {
+                    compiler = QStringLiteral("x86_arm");
+                    compilerArch = QStringLiteral("arm");
+                } else if (arch == QStringLiteral("x64")) {
+                    compiler = QStringLiteral("x86_amd64");
+                    compilerArch = QStringLiteral("amd64");
+                }
+#ifdef Q_OS_WIN
+#ifdef Q_OS_WIN64
+                const QString regKey = QStringLiteral("Software\\Wow6432Node\\Microsoft\\VisualStudio\\11.0\\Setup\\VC\\ProductDir");
+#else
+                const QString regKey = QStringLiteral("Software\\Microsoft\\VisualStudio\\11.0\\Setup\\VC\\ProductDir");
+#endif
+                const QString vcInstallDir = qt_readRegistryKey(HKEY_LOCAL_MACHINE, regKey);
+                if (vcInstallDir.isEmpty()) {
+                    fprintf(stderr, "Failed to find the Visual Studio 2012 installation directory.\n"
+                                    "Is it installed?.\n");
+                    return false;
+                }
+#else
+                const QString vcInstallDir = "/fake/vc_install_dir";
+#endif // Q_OS_WIN
+
+                QStringList incDirs;
+                QStringList libDirs;
+                QStringList binDirs;
+                const bool isPhone = project->isActiveConfig(QStringLiteral("winphone"));
+                if (isPhone) {
+                    QString sdkDir = vcInstallDir + QStringLiteral("/WPSDK/WP80");
+                    if (!QDir(sdkDir).exists()) {
+                        fprintf(stderr, "Failed to find the Windows Phone SDK in %s.\n"
+                                        "Check that it is properly installed.\n",
+                                qPrintable(QDir::toNativeSeparators(sdkDir)));
+                        return false;
+                    }
+                    incDirs << sdkDir + QStringLiteral("/include");
+                    libDirs << sdkDir + QStringLiteral("/lib/") + compilerArch;
+                    binDirs << sdkDir + QStringLiteral("/bin/") + compiler;
+
+                    QString kitDir = vcInstallDir + QStringLiteral("/../../Windows Phone Kits/8.0");
+                    if (!QDir(kitDir).exists()) {
+                        fprintf(stderr, "Failed to find the Windows Phone Kit in %s.\n"
+                                        "Check that it is properly installed.\n",
+                                qPrintable(QDir::toNativeSeparators(kitDir)));
+                        return false;
+                    }
+                    libDirs << kitDir + QStringLiteral("/lib/") + arch;
+                    incDirs << kitDir + QStringLiteral("/include")
+                            << kitDir + QStringLiteral("/include/abi")
+                            << kitDir + QStringLiteral("/include/mincore")
+                            << kitDir + QStringLiteral("/include/minwin");
+                } else {
+                    incDirs << vcInstallDir + QStringLiteral("/include");
+                    libDirs << vcInstallDir + QStringLiteral("/lib/") + compilerArch;
+                    binDirs << vcInstallDir + QStringLiteral("/bin/") + compiler
+                            << vcInstallDir + QStringLiteral("/../Common7/IDE");
+
+                    QString kitDir = vcInstallDir + QStringLiteral("/../../Windows Kits/8.0");
+                    if (!QDir(kitDir).exists()) {
+                        fprintf(stderr, "Failed to find the Windows Kit in %s.\n"
+                                        "Check that it is properly installed.\n",
+                                qPrintable(QDir::toNativeSeparators(kitDir)));
+                        return false;
+                    }
+                    libDirs << kitDir + QStringLiteral("/Lib/win8/um/") + arch;
+                    incDirs << kitDir + QStringLiteral("/include/um")
+                            << kitDir + QStringLiteral("/include/shared")
+                            << kitDir + QStringLiteral("/include/winrt");
+                }
+
+                // Inherit PATH
+                binDirs << QString::fromLocal8Bit(qgetenv("PATH")).split(QLatin1Char(';'));
+
+                t << "\nINCLUDE = " << nmakePathList(incDirs);
+                t << "\nLIB = " << nmakePathList(libDirs);
+                t << "\nPATH = " << nmakePathList(binDirs) << '\n';
             }
         }
         writeNmakeParts(t);
@@ -150,8 +235,10 @@ QString NmakeMakefileGenerator::defaultInstall(const QString &t)
     if(targetdir.right(1) != Option::dir_sep)
         targetdir += Option::dir_sep;
 
-    if(t == "target" && project->first("TEMPLATE") == "lib") {
-        if(project->isActiveConfig("shared") && project->isActiveConfig("debug")) {
+    if (project->isActiveConfig("debug")) {
+        if (t == "dlltarget"
+            || (project->first("TEMPLATE") == "lib"
+                && project->isActiveConfig("shared"))) {
             QString pdb_target = getPdbTarget();
             pdb_target.remove('"');
             QString src_targ = (project->isEmpty("DESTDIR") ? QString("$(DESTDIR)") : project->first("DESTDIR")) + pdb_target;
@@ -193,7 +280,7 @@ void NmakeMakefileGenerator::writeNmakeParts(QTextStream &t)
     if(usePCH) {
         QString precompRule = QString("-c -Yc -Fp%1 -Fo%2").arg(precompPch).arg(precompObj);
         t << precompObj << ": " << precompH << " " << escapeDependencyPaths(findDependencies(precompH)).join(" \\\n\t\t")
-          << "\n\t" << "$(CXX) " + precompRule +" $(CXXFLAGS) $(INCPATH) -TP " << precompH << endl << endl;
+          << "\n\t$(CXX) " + precompRule +" $(CXXFLAGS) $(INCPATH) -TP " << precompH << endl << endl;
     }
 }
 
@@ -359,10 +446,10 @@ void NmakeMakefileGenerator::writeImplicitRulesPart(QTextStream &t)
                 objDir = "";
             for(QStringList::Iterator cppit = Option::cpp_ext.begin(); cppit != Option::cpp_ext.end(); ++cppit)
                 t << "{" << it.key() << "}" << (*cppit) << "{" << objDir << "}" << Option::obj_ext << "::\n\t"
-                  << var("QMAKE_RUN_CXX_IMP_BATCH").replace(QRegExp("\\$@"), var("OBJECTS_DIR")) << endl << "\t$<" << endl << "<<" << endl << endl;
+                  << var("QMAKE_RUN_CXX_IMP_BATCH").replace(QRegExp("\\$@"), var("OBJECTS_DIR")) << endl << "\t$<\n<<\n\n";
             for(QStringList::Iterator cit = Option::c_ext.begin(); cit != Option::c_ext.end(); ++cit)
                 t << "{" << it.key() << "}" << (*cit) << "{" << objDir << "}" << Option::obj_ext << "::\n\t"
-                  << var("QMAKE_RUN_CC_IMP_BATCH").replace(QRegExp("\\$@"), var("OBJECTS_DIR")) << endl << "\t$<" << endl << "<<" << endl << endl;
+                  << var("QMAKE_RUN_CC_IMP_BATCH").replace(QRegExp("\\$@"), var("OBJECTS_DIR")) << endl << "\t$<\n<<\n\n";
         }
     } else {
         for(QStringList::Iterator cppit = Option::cpp_ext.begin(); cppit != Option::cpp_ext.end(); ++cppit)
@@ -377,14 +464,14 @@ void NmakeMakefileGenerator::writeBuildRulesPart(QTextStream &t)
 {
     const ProString templateName = project->first("TEMPLATE");
 
-    t << "first: all" << endl;
-    t << "all: " << fileFixify(Option::output.fileName()) << " " << varGlue("ALL_DEPS"," "," "," ") << "$(DESTDIR_TARGET)" << endl << endl;
+    t << "first: all\n";
+    t << "all: " << fileFixify(Option::output.fileName()) << " " << varGlue("ALL_DEPS"," "," "," ") << "$(DESTDIR_TARGET)\n\n";
     t << "$(DESTDIR_TARGET): " << var("PRE_TARGETDEPS") << " $(OBJECTS) " << var("POST_TARGETDEPS");
 
     if(!project->isEmpty("QMAKE_PRE_LINK"))
         t << "\n\t" <<var("QMAKE_PRE_LINK");
     if(project->isActiveConfig("staticlib")) {
-        t << "\n\t" << "$(LIBAPP) $(LIBFLAGS) " << var("QMAKE_LINK_O_FLAG") << "$(DESTDIR_TARGET) @<<" << "\n\t  "
+        t << "\n\t$(LIBAPP) $(LIBFLAGS) " << var("QMAKE_LINK_O_FLAG") << "$(DESTDIR_TARGET) @<<\n\t  "
           << "$(OBJECTS)"
           << "\n<<";
     } else if (templateName != "aux") {
@@ -433,7 +520,7 @@ void NmakeMakefileGenerator::writeBuildRulesPart(QTextStream &t)
                     writeLinkCommand(t, extraLFlags, manifest_res);
                     t << "\n\tif exist " << manifest_bak << " del " << manifest_bak;
                 } else {
-                    t << "\n\t" << "rc.exe /fo" << manifest_res << " " << manifest_rc;
+                    t << "\n\trc.exe /fo" << manifest_res << " " << manifest_rc;
                     t << "\n\t";
                     writeLinkCommand(t, extraLFlags, manifest_res);
                 }
@@ -441,7 +528,7 @@ void NmakeMakefileGenerator::writeBuildRulesPart(QTextStream &t)
                 // directly embed the manifest in the executable after linking
                 t << "\n\t";
                 writeLinkCommand(t, extraLFlags);
-                t << "\n\t" << "mt.exe /nologo /manifest " << manifest
+                t << "\n\tmt.exe /nologo /manifest " << manifest
                   << " /outputresource:$(DESTDIR_TARGET);" << resourceId;
             }
         }  else {
