@@ -76,6 +76,10 @@
 #include <CoreServices/CoreServices.h>
 #endif
 
+#if defined(Q_OS_ANDROID)
+#include <private/qjni_p.h>
+#endif
+
 QT_BEGIN_NAMESPACE
 
 #if !QT_DEPRECATED_SINCE(5, 0)
@@ -84,6 +88,14 @@ Q_CORE_EXPORT void *qMemCopy(void *dest, const void *src, size_t n);
 Q_CORE_EXPORT void *qMemSet(void *dest, int c, size_t n);
 #endif
 
+// Statically check assumptions about the environment we're running
+// in. The idea here is to error or warn if otherwise implicit Qt
+// assumptions are not fulfilled on new hardware or compilers
+// (if this list becomes too long, consider factoring into a separate file)
+Q_STATIC_ASSERT_X(sizeof(int) == 4, "Qt assumes that int is 32 bits");
+Q_STATIC_ASSERT_X(UCHAR_MAX == 255, "Qt assumes that char is 8 bits");
+
+
 /*!
     \class QFlag
     \inmodule QtCore
@@ -346,15 +358,15 @@ Q_CORE_EXPORT void *qMemSet(void *dest, int c, size_t n);
 /*!
     \fn bool QFlags::operator!() const
 
-    Returns true if no flag is set (i.e., if the value stored by the
-    QFlags object is 0); otherwise returns false.
+    Returns \c true if no flag is set (i.e., if the value stored by the
+    QFlags object is 0); otherwise returns \c false.
 */
 
 /*!
     \fn bool QFlags::testFlag(Enum flag) const
     \since 4.2
 
-    Returns true if the \a flag is set, otherwise false.
+    Returns \c true if the \a flag is set, otherwise false.
 */
 
 /*!
@@ -511,7 +523,7 @@ Q_CORE_EXPORT void *qMemSet(void *dest, int c, size_t n);
     the application is compiled using Forte Developer, or Sun Studio
     C++.  The header file also declares a range of macros (Q_OS_*)
     that are defined for the specified platforms. For example,
-    Q_OS_X11 which is defined for the X Window System.
+    Q_OS_UNIX which is defined for the Unix-based systems.
 
     The purpose of these macros is to enable programmers to add
     compiler or platform specific code to their application.
@@ -544,10 +556,8 @@ Q_CORE_EXPORT void *qMemSet(void *dest, int c, size_t n);
     \typedef qreal
     \relates <QtGlobal>
 
-    Typedef for \c double on all platforms except for those using CPUs with
-    ARM architectures.
-    On ARM-based platforms, \c qreal is a typedef for \c float for performance
-    reasons.
+    Typedef for \c double unless Qt is configured with the
+    \c{-qreal float} option.
 */
 
 /*! \typedef uchar
@@ -934,7 +944,7 @@ bool qSharedBuild() Q_DECL_NOTHROW
     \endlist
 
     Some constants are defined only on certain platforms. You can use
-    the preprocessor symbols Q_OS_WIN and Q_OS_MACX to test that
+    the preprocessor symbols Q_OS_WIN and Q_OS_OSX to test that
     the application is compiled under Windows or OS X.
 
     \sa QLibraryInfo
@@ -973,8 +983,7 @@ bool qSharedBuild() Q_DECL_NOTHROW
 /*!
     \fn QSysInfo::MacVersion QSysInfo::macVersion()
 
-    Returns the version of Mac OS X on which the application is run (Mac OS X
-    Only).
+    Returns the version of Darwin (OS X or iOS) on which the application is run.
 */
 
 /*!
@@ -1010,6 +1019,7 @@ bool qSharedBuild() Q_DECL_NOTHROW
     \value WV_VISTA Windows Vista, Windows Server 2008 (operating system version 6.0)
     \value WV_WINDOWS7 Windows 7, Windows Server 2008 R2 (operating system version 6.1)
     \value WV_WINDOWS8 Windows 8 (operating system version 6.2)
+    \value WV_WINDOWS8_1 Windows 8.1 (operating system version 6.3), introduced in Qt 5.2
 
     Alternatively, you may use the following macros which correspond directly to the Windows operating system version number:
 
@@ -1020,6 +1030,7 @@ bool qSharedBuild() Q_DECL_NOTHROW
     \value WV_6_0   Operating system version 6.0, corresponds to Windows Vista and Windows Server 2008
     \value WV_6_1   Operating system version 6.1, corresponds to Windows 7 and Windows Server 2008 R2
     \value WV_6_2   Operating system version 6.2, corresponds to Windows 8
+    \value WV_6_3   Operating system version 6.3, corresponds to Windows 8.1, introduced in Qt 5.2
 
     CE-based versions:
 
@@ -1042,7 +1053,7 @@ bool qSharedBuild() Q_DECL_NOTHROW
     \enum QSysInfo::MacVersion
 
     This enum provides symbolic names for the various versions of the
-    OS X operating system. On OS X, the
+    Darwin operating system, covering both OS X and iOS. The
     QSysInfo::MacintoshVersion variable gives the version of the
     system on which the application is run.
 
@@ -1070,6 +1081,15 @@ bool qSharedBuild() Q_DECL_NOTHROW
     \value MV_MOUNTAINLION Apple codename for MV_10_8
     \value MV_MAVERICKS    Apple codename for MV_10_9
 
+    \value MV_IOS      iOS (any)
+    \value MV_IOS_4_3  iOS 4.3
+    \value MV_IOS_5_0  iOS 5.0
+    \value MV_IOS_5_1  iOS 5.1
+    \value MV_IOS_6_0  iOS 6.0
+    \value MV_IOS_6_1  iOS 6.1
+    \value MV_IOS_7_0  iOS 7.0
+    \value MV_IOS_7_1  iOS 7.1
+
     \sa WinVersion
 */
 
@@ -1077,18 +1097,20 @@ bool qSharedBuild() Q_DECL_NOTHROW
     \macro Q_OS_DARWIN
     \relates <QtGlobal>
 
-    Defined on Darwin OS (synonym for Q_OS_MAC).
+    Defined on Darwin-based operating systems such as OS X and iOS,
+    including any open source version(s) of Darwin.
 */
 
 /*!
     \macro Q_OS_MAC
     \relates <QtGlobal>
 
-    Defined on OS X and iOS (synonym for Q_OS_DARWIN).
+    Defined on Darwin-based operating systems distributed by Apple, which
+    currently includes OS X and iOS, but not the open source version.
  */
 
 /*!
-    \macro Q_OS_MACX
+    \macro Q_OS_OSX
     \relates <QtGlobal>
 
     Defined on OS X.
@@ -1130,6 +1152,21 @@ bool qSharedBuild() Q_DECL_NOTHROW
     Defined on Windows CE.
 */
 
+/*!
+    \macro Q_OS_WINRT
+    \relates <QtGlobal>
+
+    Defined for Windows Runtime (Windows Store apps) on Windows 8, Windows RT,
+    and Windows Phone 8.
+*/
+
+/*!
+    \macro Q_OS_WINPHONE
+    \relates <QtGlobal>
+
+    Defined on Windows Phone 8.
+*/
+
 /*!
     \macro Q_OS_CYGWIN
     \relates <QtGlobal>
@@ -1165,6 +1202,13 @@ bool qSharedBuild() Q_DECL_NOTHROW
     Defined on Linux.
 */
 
+/*!
+    \macro Q_OS_ANDROID
+    \relates <QtGlobal>
+
+    Defined on Android.
+*/
+
 /*!
     \macro Q_OS_FREEBSD
     \relates <QtGlobal>
@@ -1668,13 +1712,15 @@ static const unsigned int qt_one = 1;
 const int QSysInfo::ByteOrder = ((*((unsigned char *) &qt_one) == 0) ? BigEndian : LittleEndian);
 #endif
 
-#if defined(Q_OS_MACX)
+#if defined(Q_OS_MAC)
 
 QT_BEGIN_INCLUDE_NAMESPACE
 #include "private/qcore_mac_p.h"
 #include "qnamespace.h"
 QT_END_INCLUDE_NAMESPACE
 
+#if defined(Q_OS_OSX)
+
 Q_CORE_EXPORT OSErr qt_mac_create_fsref(const QString &file, FSRef *fsref)
 {
     return FSPathMakeRef(reinterpret_cast<const UInt8 *>(file.toUtf8().constData()), fsref, 0);
@@ -1690,28 +1736,60 @@ Q_CORE_EXPORT void qt_mac_to_pascal_string(QString s, Str255 str, TextEncoding e
 Q_CORE_EXPORT QString qt_mac_from_pascal_string(const Str255 pstr) {
     return QCFString(CFStringCreateWithPascalString(0, pstr, CFStringGetSystemEncoding()));
 }
-#endif // defined(Q_OS_MACX)
-
-#if defined(Q_OS_MAC)
+#endif // defined(Q_OS_OSX)
 
 QSysInfo::MacVersion QSysInfo::macVersion()
 {
-#ifndef Q_OS_IOS
+#if defined(Q_OS_OSX)
     SInt32 gestalt_version;
     if (Gestalt(gestaltSystemVersion, &gestalt_version) == noErr) {
         return QSysInfo::MacVersion(((gestalt_version & 0x00F0) >> 4) + 2);
     }
+#elif defined(Q_OS_IOS)
+    return qt_ios_version(); // qtcore_mac_objc.mm
 #endif
     return QSysInfo::MV_Unknown;
 }
 const QSysInfo::MacVersion QSysInfo::MacintoshVersion = QSysInfo::macVersion();
 
-#elif defined(Q_OS_WIN) || defined(Q_OS_CYGWIN) || defined(Q_OS_WINCE)
+#elif defined(Q_OS_WIN) || defined(Q_OS_CYGWIN) || defined(Q_OS_WINCE) || defined(Q_OS_WINRT)
 
 QT_BEGIN_INCLUDE_NAMESPACE
 #include "qt_windows.h"
 QT_END_INCLUDE_NAMESPACE
 
+#ifndef Q_OS_WINRT
+static inline OSVERSIONINFO winOsVersion()
+{
+    OSVERSIONINFO result = { sizeof(OSVERSIONINFO), 0, 0, 0, 0, {'\0'}};
+    // GetVersionEx() has been deprecated in Windows 8.1 and will return
+    // only Windows 8 from that version on.
+#  if defined(_MSC_VER) && _MSC_VER >= 1800
+#    pragma warning( push )
+#    pragma warning( disable : 4996 )
+#  endif
+    GetVersionEx(&result);
+#  if defined(_MSC_VER) && _MSC_VER >= 1800
+#    pragma warning( pop )
+#  endif
+#  ifndef Q_OS_WINCE
+    if (result.dwMajorVersion == 6 && result.dwMinorVersion == 2) {
+        // This could be Windows 8.1 or higher. Note that as of Windows 9,
+        // the major version needs to be checked as well.
+        DWORDLONG conditionMask = 0;
+        VER_SET_CONDITION(conditionMask, VER_MAJORVERSION, VER_GREATER_EQUAL);
+        VER_SET_CONDITION(conditionMask, VER_MINORVERSION, VER_GREATER_EQUAL);
+        VER_SET_CONDITION(conditionMask, VER_PLATFORMID, VER_EQUAL);
+        OSVERSIONINFOEX checkVersion = { sizeof(OSVERSIONINFOEX), result.dwMajorVersion, result.dwMinorVersion,
+                                         result.dwBuildNumber, result.dwPlatformId, {'\0'}, 0, 0, 0, 0, 0 };
+        for ( ; VerifyVersionInfo(&checkVersion, VER_MAJORVERSION | VER_MINORVERSION | VER_PLATFORMID, conditionMask); ++checkVersion.dwMinorVersion)
+            result.dwMinorVersion = checkVersion.dwMinorVersion;
+    }
+#  endif // !Q_OS_WINCE
+    return result;
+}
+#endif // !Q_OS_WINRT
+
 QSysInfo::WinVersion QSysInfo::windowsVersion()
 {
 #ifndef VER_PLATFORM_WIN32s
@@ -1730,10 +1808,11 @@ QSysInfo::WinVersion QSysInfo::windowsVersion()
     static QSysInfo::WinVersion winver;
     if (winver)
         return winver;
+#ifdef Q_OS_WINRT
+    winver = QSysInfo::WV_WINDOWS8;
+#else
     winver = QSysInfo::WV_NT;
-    OSVERSIONINFO osver;
-    osver.dwOSVersionInfoSize = sizeof(osver);
-    GetVersionEx(&osver);
+    const OSVERSIONINFO osver = winOsVersion();
 #ifdef Q_OS_WINCE
     DWORD qt_cever = 0;
     qt_cever = osver.dwMajorVersion * 100;
@@ -1779,6 +1858,8 @@ QSysInfo::WinVersion QSysInfo::windowsVersion()
             winver = QSysInfo::WV_WINDOWS7;
         } else if (osver.dwMajorVersion == 6 && osver.dwMinorVersion == 2) {
             winver = QSysInfo::WV_WINDOWS8;
+        } else if (osver.dwMajorVersion == 6 && osver.dwMinorVersion == 3) {
+            winver = QSysInfo::WV_WINDOWS8_1;
         } else {
             qWarning("Qt: Untested Windows version %d.%d detected!",
                      int(osver.dwMajorVersion), int(osver.dwMinorVersion));
@@ -1814,6 +1895,7 @@ QSysInfo::WinVersion QSysInfo::windowsVersion()
             winver = QSysInfo::WV_WINDOWS8;
     }
 #endif
+#endif // !Q_OS_WINRT
 
     return winver;
 }
@@ -2066,11 +2148,11 @@ namespace {
     // version in portable code. However, it's impossible to do that if
     // _GNU_SOURCE is defined so we use C++ overloading to decide what to do
     // depending on the return type
-    static inline QString fromstrerror_helper(int, const QByteArray &buf)
+    static inline Q_DECL_UNUSED QString fromstrerror_helper(int, const QByteArray &buf)
     {
         return QString::fromLocal8Bit(buf);
     }
-    static inline QString fromstrerror_helper(const char *str, const QByteArray &)
+    static inline Q_DECL_UNUSED QString fromstrerror_helper(const char *str, const QByteArray &)
     {
         return QString::fromLocal8Bit(str);
     }
@@ -2263,7 +2345,7 @@ bool qputenv(const char *varName, const QByteArray& value)
 
     This function deletes the variable \a varName from the environment.
 
-    Returns true on success.
+    Returns \c true on success.
 
     \since 5.1
 
@@ -2291,7 +2373,7 @@ bool qunsetenv(const char *varName)
 #endif
 }
 
-#if defined(Q_OS_UNIX) && !defined(QT_NO_THREAD)
+#if defined(Q_OS_UNIX) && !defined(QT_NO_THREAD) && defined(_POSIX_THREAD_SAFE_FUNCTIONS) && (_POSIX_THREAD_SAFE_FUNCTIONS - 0 > 0)
 
 #  if defined(Q_OS_INTEGRITY) && defined(__GHS_VERSION_NUMBER) && (__GHS_VERSION_NUMBER < 500)
 // older versions of INTEGRITY used a long instead of a uint for the seed.
@@ -2303,6 +2385,9 @@ typedef uint SeedStorageType;
 typedef QThreadStorage<SeedStorageType *> SeedStorage;
 Q_GLOBAL_STATIC(SeedStorage, randTLS)  // Thread Local Storage for seed value
 
+#elif defined(Q_OS_ANDROID)
+typedef QThreadStorage<QJNIObjectPrivate> AndroidRandomStorage;
+Q_GLOBAL_STATIC(AndroidRandomStorage, randomTLS)
 #endif
 
 /*!
@@ -2322,7 +2407,7 @@ Q_GLOBAL_STATIC(SeedStorage, randTLS)  // Thread Local Storage for seed value
 */
 void qsrand(uint seed)
 {
-#if defined(Q_OS_UNIX) && !defined(QT_NO_THREAD)
+#if defined(Q_OS_UNIX) && !defined(QT_NO_THREAD) && defined(_POSIX_THREAD_SAFE_FUNCTIONS) && (_POSIX_THREAD_SAFE_FUNCTIONS - 0 > 0)
     SeedStorage *seedStorage = randTLS();
     if (seedStorage) {
         SeedStorageType *pseed = seedStorage->localData();
@@ -2336,6 +2421,21 @@ void qsrand(uint seed)
         //global static object, fallback to srand(seed)
         srand(seed);
     }
+#elif defined(Q_OS_ANDROID)
+    if (randomTLS->hasLocalData()) {
+        randomTLS->localData().callMethod<void>("setSeed", "(J)V", jlong(seed));
+        return;
+    }
+
+    QJNIObjectPrivate random("java/util/Random",
+                             "(J)V",
+                             jlong(seed));
+    if (!random.isValid()) {
+        srand(seed);
+        return;
+    }
+
+    randomTLS->setLocalData(random);
 #else
     // On Windows srand() and rand() already use Thread-Local-Storage
     // to store the seed between calls
@@ -2377,6 +2477,26 @@ int qrand()
         //global static object, fallback to rand()
         return rand();
     }
+#elif defined(Q_OS_ANDROID)
+    AndroidRandomStorage *randomStorage = randomTLS();
+    if (!randomStorage)
+        return rand();
+
+    if (randomStorage->hasLocalData()) {
+        return randomStorage->localData().callMethod<jint>("nextInt",
+                                                           "(I)I",
+                                                           RAND_MAX);
+    }
+
+    QJNIObjectPrivate random("java/util/Random",
+                             "(J)V",
+                             jlong(1));
+
+    if (!random.isValid())
+        return rand();
+
+    randomStorage->setLocalData(random);
+    return random.callMethod<jint>("nextInt", "(I)I", RAND_MAX);
 #else
     // On Windows srand() and rand() already use Thread-Local-Storage
     // to store the seed between calls
@@ -3333,4 +3453,36 @@ bool QInternal::activateCallbacks(Callback cb, void **parameters)
     \sa Q_DECL_OVERRIDE
 */
 
+/*!
+    \macro Q_FORWARD_DECLARE_OBJC_CLASS(classname)
+    \since 5.2
+    \relates <QtGlobal>
+
+    Forward-declares an Objective-C \a classname in a manner such that it can be
+    compiled as either Objective-C or C++.
+
+    This is primarily intended for use in header files that may be included by
+    both Objective-C and C++ source files.
+*/
+
+/*!
+    \macro Q_FORWARD_DECLARE_CF_TYPE(type)
+    \since 5.2
+    \relates <QtGlobal>
+
+    Forward-declares a Core Foundation \a type. This includes the actual
+    type and the ref type. For example, Q_FORWARD_DECLARE_CF_TYPE(CFString)
+    declares __CFString and CFStringRef.
+*/
+
+/*!
+    \macro Q_FORWARD_DECLARE_MUTABLE_CF_TYPE(type)
+    \since 5.2
+    \relates <QtGlobal>
+
+    Forward-declares a mutable Core Foundation \a type. This includes the actual
+    type and the ref type. For example, Q_FORWARD_DECLARE_CF_TYPE(CFString)
+    declares __CFMutableString and CFMutableStringRef.
+*/
+
 QT_END_NAMESPACE
