@@ -53,9 +53,11 @@
 
 #include <QtGui/private/qguiapplication_p.h>
 #include <qpa/qplatformfontdatabase.h>
+#include <qpa/qplatformtheme.h>
 
 #include <stdlib.h>
 #include <limits.h>
+#include <algorithm>
 
 
 // #define QFONTDATABASE_DEBUG
@@ -314,9 +316,6 @@ struct  QtFontFamily
     QtFontFamily(const QString &n)
         :
         fixedPitch(false),
-#if !defined(QWS) && defined(Q_OS_MAC)
-        fixedPitchComputed(false),
-#endif
         name(n), count(0), foundries(0)
         , bogusWritingSystems(false)
         , askedForFallback(false)
@@ -330,9 +329,6 @@ struct  QtFontFamily
     }
 
     bool fixedPitch : 1;
-#if !defined(QWS) && defined(Q_OS_MAC)
-    bool fixedPitchComputed : 1;
-#endif
 
     QString name;
     QStringList aliases;
@@ -348,18 +344,6 @@ struct  QtFontFamily
     QtFontFoundry *foundry(const QString &f, bool = false);
 };
 
-#if !defined(QWS) && defined(Q_OS_MAC)
-inline static void qt_mac_get_fixed_pitch(QtFontFamily *f)
-{
-    if(f && !f->fixedPitchComputed) {
-        QFontMetrics fm(f->name);
-        f->fixedPitch = fm.width(QLatin1Char('i')) == fm.width(QLatin1Char('m'));
-        f->fixedPitchComputed = true;
-    }
-}
-#endif
-
-
 QtFontFoundry *QtFontFamily::foundry(const QString &f, bool create)
 {
     if (f.isNull() && count == 1)
@@ -823,9 +807,6 @@ unsigned int bestFoundry(int script, unsigned int score, int styleStrategy,
             EncodingMismatch    = 0x0002
         };
         if (pitch != '*') {
-#if !defined(QWS) && defined(Q_OS_MAC)
-            qt_mac_get_fixed_pitch(const_cast<QtFontFamily*>(family));
-#endif
             if ((pitch == 'm' && !family->fixedPitch)
                 || (pitch == 'p' && family->fixedPitch))
                 this_score += PitchMismatch;
@@ -1102,6 +1083,17 @@ QFontDatabase::QFontDatabase()
     \omitvalue WritingSystemsCount
 */
 
+/*!
+    \enum QFontDatabase::SystemFont
+
+    \value GeneralFont              The default system font.
+    \value FixedFont                The fixed font that the system recommends.
+    \value TitleFont                The system standard font for titles.
+    \value SmallestReadableFont     The smallest readable system font.
+
+    \since 5.2
+*/
+
 /*!
     Returns a sorted list of the available writing systems. This is
     list generated from information about all installed fonts on the
@@ -1128,7 +1120,7 @@ QList<QFontDatabase::WritingSystem> QFontDatabase::writingSystems() const
                 list.append(writingSystem);
         }
     }
-    qSort(list);
+    std::sort(list.begin(), list.end());
     return list;
 }
 
@@ -1246,8 +1238,8 @@ QStringList QFontDatabase::styles(const QString &family) const
 }
 
 /*!
-    Returns true if the font that has family \a family and style \a
-    style is fixed pitch; otherwise returns false.
+    Returns \c true if the font that has family \a family and style \a
+    style is fixed pitch; otherwise returns \c false.
 */
 
 bool QFontDatabase::isFixedPitch(const QString &family,
@@ -1263,15 +1255,12 @@ bool QFontDatabase::isFixedPitch(const QString &family,
     QT_PREPEND_NAMESPACE(load)(familyName);
 
     QtFontFamily *f = d->family(familyName);
-#if !defined(QWS) && defined(Q_OS_MAC)
-    qt_mac_get_fixed_pitch(f);
-#endif
     return (f && f->fixedPitch);
 }
 
 /*!
-    Returns true if the font that has family \a family and style \a
-    style is a scalable bitmap font; otherwise returns false. Scaling
+    Returns \c true if the font that has family \a family and style \a
+    style is a scalable bitmap font; otherwise returns \c false. Scaling
     a bitmap font usually produces an unattractive hardly readable
     result, because the pixels of the font are scaled. If you need to
     scale a bitmap font it is better to scale it to one of the fixed
@@ -1314,9 +1303,9 @@ bool QFontDatabase::isBitmapScalable(const QString &family,
 
 
 /*!
-    Returns true if the font that has family \a family and style \a
-    style is smoothly scalable; otherwise returns false. If this
-    function returns true, it's safe to scale this font to any size,
+    Returns \c true if the font that has family \a family and style \a
+    style is smoothly scalable; otherwise returns \c false. If this
+    function returns \c true, it's safe to scale this font to any size,
     and the result will always look attractive.
 
     \sa isScalable(), isBitmapScalable()
@@ -1353,8 +1342,8 @@ bool QFontDatabase::isSmoothlyScalable(const QString &family, const QString &sty
 }
 
 /*!
-    Returns true if the font that has family \a family and style \a
-    style is scalable; otherwise returns false.
+    Returns \c true if the font that has family \a family and style \a
+    style is scalable; otherwise returns \c false.
 
     \sa isBitmapScalable(), isSmoothlyScalable()
 */
@@ -1423,7 +1412,7 @@ QList<int> QFontDatabase::pointSizes(const QString &family,
     if (smoothScalable)
         return standardSizes();
 
-    qSort(sizes);
+    std::sort(sizes.begin(), sizes.end());
     return sizes;
 }
 
@@ -1526,7 +1515,7 @@ QList<int> QFontDatabase::smoothSizes(const QString &family,
     if (smoothScalable)
         return QFontDatabase::standardSizes();
 
-    qSort(sizes);
+    std::sort(sizes.begin(), sizes.end());
     return sizes;
 }
 
@@ -1543,8 +1532,8 @@ QList<int> QFontDatabase::standardSizes()
 
 
 /*!
-    Returns true if the font that has family \a family and style \a
-    style is italic; otherwise returns false.
+    Returns \c true if the font that has family \a family and style \a
+    style is italic; otherwise returns \c false.
 
     \sa weight(), bold()
 */
@@ -1576,8 +1565,8 @@ bool QFontDatabase::italic(const QString &family, const QString &style) const
 
 
 /*!
-    Returns true if the font that has family \a family and style \a
-    style is bold; otherwise returns false.
+    Returns \c true if the font that has family \a family and style \a
+    style is bold; otherwise returns \c false.
 
     \sa italic(), weight()
 */
@@ -2113,13 +2102,50 @@ QStringList QFontDatabase::applicationFontFamilies(int id)
     return privateDb()->applicationFonts.value(id).families;
 }
 
+/*!
+    \since 5.2
+
+    Returns the most adequate font for a given \a type case for proper integration
+    with the system's look and feel.
+
+    \sa QGuiApplication::font()
+*/
+
+QFont QFontDatabase::systemFont(QFontDatabase::SystemFont type)
+{
+    const QFont *font = 0;
+    if (const QPlatformTheme *theme = QGuiApplicationPrivate::platformTheme()) {
+        switch (type) {
+            case GeneralFont:
+                font = theme->font(QPlatformTheme::SystemFont);
+                break;
+            case FixedFont:
+                font = theme->font(QPlatformTheme::FixedFont);
+                break;
+            case TitleFont:
+                font = theme->font(QPlatformTheme::TitleBarFont);
+                break;
+            case SmallestReadableFont:
+                font = theme->font(QPlatformTheme::MiniFont);
+                break;
+        }
+    }
+
+    if (font)
+        return *font;
+    else if (QPlatformIntegration *integration = QGuiApplicationPrivate::platformIntegration())
+        return integration->fontDatabase()->defaultFont();
+    else
+        return QFont();
+}
+
 /*!
     \fn bool QFontDatabase::removeApplicationFont(int id)
     \since 4.2
 
     Removes the previously loaded application font identified by \a
-    id. Returns true if unloading of the font succeeded; otherwise
-    returns false.
+    id. Returns \c true if unloading of the font succeeded; otherwise
+    returns \c false.
 
     \sa removeAllApplicationFonts(), addApplicationFont(),
         addApplicationFontFromData()
@@ -2132,8 +2158,8 @@ QStringList QFontDatabase::applicationFontFamilies(int id)
     Removes all application-local fonts previously added using addApplicationFont()
     and addApplicationFontFromData().
 
-    Returns true if unloading of the fonts succeeded; otherwise
-    returns false.
+    Returns \c true if unloading of the fonts succeeded; otherwise
+    returns \c false.
 
     \sa removeApplicationFont(), addApplicationFont(), addApplicationFontFromData()
 */
@@ -2141,12 +2167,15 @@ QStringList QFontDatabase::applicationFontFamilies(int id)
 /*!
     \fn bool QFontDatabase::supportsThreadedFontRendering()
     \since 4.4
+    \deprecated
 
-    Returns true if font rendering is supported outside the GUI
+    Returns \c true if font rendering is supported outside the GUI
     thread, false otherwise. In other words, a return value of false
     means that all QPainter::drawText() calls outside the GUI thread
     will not produce readable output.
 
+    As of 5.0, always returns \c true.
+
     \sa {Thread-Support in Qt Modules#Painting In Threads}{Painting In Threads}
 */
 
