@@ -107,12 +107,11 @@ static QTouchDevice *touchDevice = 0;
     m_maskImage = 0;
     m_maskData = 0;
     m_window = 0;
-    if (m_subscribesForGlobalFrameNotifications) {
-        m_subscribesForGlobalFrameNotifications = false;
-        [[NSNotificationCenter defaultCenter] removeObserver:self
-             name:NSViewGlobalFrameDidChangeNotification
-             object:self];
-}
+    m_subscribesForGlobalFrameNotifications = false;
+    [[NSNotificationCenter defaultCenter] removeObserver:self];
+
+    delete currentCustomDragTypes;
+
     [super dealloc];
 }
 
@@ -176,7 +175,7 @@ static QTouchDevice *touchDevice = 0;
 - (void) globalFrameChanged:(NSNotification*)notification
 {
     Q_UNUSED(notification);
-    QWindowSystemInterface::handleExposeEvent(m_window, m_window->geometry());
+    m_platformWindow->updateExposedGeometry();
 }
 
 - (void)viewDidMoveToSuperview
@@ -187,7 +186,7 @@ static QTouchDevice *touchDevice = 0;
     if ([self superview]) {
         m_platformWindow->m_contentViewIsEmbedded = true;
         QWindowSystemInterface::handleGeometryChange(m_window, m_platformWindow->geometry());
-        QWindowSystemInterface::handleExposeEvent(m_window, m_platformWindow->geometry());
+        m_platformWindow->updateExposedGeometry();
         QWindowSystemInterface::flushWindowSystemEvents();
     } else {
         m_platformWindow->m_contentViewIsEmbedded = false;
@@ -205,10 +204,26 @@ static QTouchDevice *touchDevice = 0;
                                               selector:@selector(windowNotification:)
                                               name:nil // Get all notifications
                                               object:newWindow];
-    } else {
-        [[NSNotificationCenter defaultCenter] removeObserver:self name:nil object:[self window]];
     }
+    if ([self window])
+        [[NSNotificationCenter defaultCenter] removeObserver:self name:nil object:[self window]];
 }
+
+- (QWindow *)topLevelWindow
+{
+    QWindow *focusWindow = m_window;
+
+    // For widgets we need to do a bit of trickery as the window
+    // to activate is the window of the top-level widget.
+    if (m_window->metaObject()->className() == QStringLiteral("QWidgetWindow")) {
+        while (focusWindow->parent()) {
+            focusWindow = focusWindow->parent();
+        }
+    }
+
+    return focusWindow;
+}
+
 - (void)updateGeometry
 {
     QRect geometry;
@@ -246,7 +261,7 @@ static QTouchDevice *touchDevice = 0;
     // Send a geometry change event to Qt, if it's ready to handle events
     if (!m_platformWindow->m_inConstructor) {
         QWindowSystemInterface::handleGeometryChange(m_window, geometry);
-        QWindowSystemInterface::handleExposeEvent(m_window, geometry);
+        m_platformWindow->updateExposedGeometry();
         QWindowSystemInterface::flushWindowSystemEvents();
     }
 }
@@ -283,6 +298,10 @@ static QTouchDevice *touchDevice = 0;
         Qt::WindowState newState = notificationName == NSWindowDidMiniaturizeNotification ?
                     Qt::WindowMinimized : Qt::WindowNoState;
         [self notifyWindowStateChanged:newState];
+        // NSWindowDidOrderOnScreenAndFinishAnimatingNotification is private API, and not
+        // emitted in 10.6, so we bring back the old behavior for that case alone.
+        if (newState == Qt::WindowNoState && QSysInfo::QSysInfo::MacintoshVersion == QSysInfo::MV_10_6)
+            m_platformWindow->exposeWindow();
     } else if ([notificationName isEqualToString: @"NSWindowDidOrderOffScreenNotification"]) {
         m_platformWindow->obscureWindow();
     } else if ([notificationName isEqualToString: @"NSWindowDidOrderOnScreenAndFinishAnimatingNotification"]) {
@@ -454,16 +473,7 @@ static QTouchDevice *touchDevice = 0;
 {
     if (m_window->flags() & Qt::WindowTransparentForInput)
         return NO;
-    QWindow *focusWindow = m_window;
-
-    // For widgets we need to do a bit of trickery as the window
-    // to activate is the window of the top-level widget.
-    if (m_window->metaObject()->className() == QStringLiteral("QWidgetWindow")) {
-        while (focusWindow->parent()) {
-            focusWindow = focusWindow->parent();
-        }
-    }
-    QWindowSystemInterface::handleWindowActivated(focusWindow);
+    QWindowSystemInterface::handleWindowActivated([self topLevelWindow]);
     return YES;
 }
 
@@ -486,7 +496,7 @@ static QTouchDevice *touchDevice = 0;
     return YES;
 }
 
-- (void)convertFromEvent:(NSEvent *)event toWindowPoint:(QPoint *)qtWindowPoint andScreenPoint:(QPoint *)qtScreenPoint
+- (void)convertFromScreen:(NSPoint)mouseLocation toWindowPoint:(QPointF *)qtWindowPoint andScreenPoint:(QPointF *)qtScreenPoint
 {
     // Calculate the mouse position in the QWindow and Qt screen coordinate system,
     // starting from coordinates in the NSWindow coordinate system.
@@ -506,23 +516,22 @@ static QTouchDevice *touchDevice = 0;
     // NSView and QWindow are equal coordinate systems: the QWindow covers the
     // entire NSView, and we've set the NSView's isFlipped property to true.
 
-    NSPoint nsWindowPoint = [event locationInWindow];                    // NSWindow coordinates
-
-    NSPoint nsViewPoint = [self convertPoint: nsWindowPoint fromView: nil]; // NSView/QWindow coordinates
-    *qtWindowPoint = QPoint(nsViewPoint.x, nsViewPoint.y);                     // NSView/QWindow coordinates
-
     NSWindow *window = [self window];
+    NSPoint nsWindowPoint;
     // Use convertRectToScreen if available (added in 10.7).
 #if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_7
-    if ([window respondsToSelector:@selector(convertRectToScreen:)]) {
-        NSRect screenRect = [window convertRectToScreen : NSMakeRect(nsWindowPoint.x, nsWindowPoint.y, 0, 0)]; // OS X screen coordinates
-        *qtScreenPoint = QPoint(screenRect.origin.x, qt_mac_flipYCoordinate(screenRect.origin.y));              // Qt screen coordinates
+    if ([window respondsToSelector:@selector(convertRectFromScreen:)]) {
+        NSRect windowRect = [window convertRectFromScreen:NSMakeRect(mouseLocation.x, mouseLocation.y, 1, 1)];
+        nsWindowPoint = windowRect.origin;                    // NSWindow coordinates
     } else
 #endif
     {
-        NSPoint screenPoint = [window convertBaseToScreen : NSMakePoint(nsWindowPoint.x, nsWindowPoint.y)];
-        *qtScreenPoint = QPoint(screenPoint.x, qt_mac_flipYCoordinate(screenPoint.y));
+        nsWindowPoint = [window convertScreenToBase:mouseLocation];                    // NSWindow coordinates
     }
+    NSPoint nsViewPoint = [self convertPoint: nsWindowPoint fromView: nil]; // NSView/QWindow coordinates
+    *qtWindowPoint = QPointF(nsViewPoint.x, nsViewPoint.y);                     // NSView/QWindow coordinates
+
+    *qtScreenPoint = QPointF(mouseLocation.x, qt_mac_flipYCoordinate(mouseLocation.y)); // Qt screen coordinates
 }
 
 - (void)resetMouseButtons
@@ -532,8 +541,11 @@ static QTouchDevice *touchDevice = 0;
 
 - (void)handleMouseEvent:(NSEvent *)theEvent
 {
-    QPoint qtWindowPoint, qtScreenPoint;
-    [self convertFromEvent:theEvent toWindowPoint:&qtWindowPoint andScreenPoint:&qtScreenPoint];
+    [self handleTabletEvent: theEvent];
+
+    QPointF qtWindowPoint;
+    QPointF qtScreenPoint;
+    [self convertFromScreen:[NSEvent mouseLocation] toWindowPoint:&qtWindowPoint andScreenPoint:&qtScreenPoint];
     ulong timestamp = [theEvent timestamp] * 1000;
 
     QCocoaDrag* nativeDrag = static_cast<QCocoaDrag *>(QGuiApplicationPrivate::platformIntegration()->drag());
@@ -594,6 +606,7 @@ static QTouchDevice *touchDevice = 0;
         QWindowSystemInterface::handleCloseEvent(m_platformWindow->m_activePopupWindow);
         QWindowSystemInterface::flushWindowSystemEvents();
         m_platformWindow->m_activePopupWindow = 0;
+        return;
     }
     if ([self hasMarkedText]) {
         NSInputManager* inputManager = [NSInputManager currentInputManager];
@@ -658,7 +671,7 @@ static QTouchDevice *touchDevice = 0;
     // mouse moves delivered to it (Apple recommends keeping it OFF because there
     // is a performance hit). So it goes.
     NSUInteger trackingOptions = NSTrackingMouseEnteredAndExited | NSTrackingActiveInActiveApp
-                                 | NSTrackingInVisibleRect | NSTrackingMouseMoved;
+                                 | NSTrackingInVisibleRect | NSTrackingMouseMoved | NSTrackingCursorUpdate;
     NSTrackingArea *ta = [[[NSTrackingArea alloc] initWithRect:[self frame]
                                                       options:trackingOptions
                                                         owner:self
@@ -667,14 +680,22 @@ static QTouchDevice *touchDevice = 0;
     [self addTrackingArea:ta];
 }
 
+-(void)cursorUpdate:(NSEvent *)theEvent
+{
+    Q_UNUSED(theEvent)
+    if (m_platformWindow->m_windowCursor)
+        [m_platformWindow->m_windowCursor set];
+}
+
 - (void)mouseMoved:(NSEvent *)theEvent
 {
     if (m_window->flags() & Qt::WindowTransparentForInput)
         return [super mouseMoved:theEvent];
 
-    QPoint windowPoint, screenPoint;
-    [self convertFromEvent:theEvent toWindowPoint:&windowPoint andScreenPoint:&screenPoint];
-    QWindow *childWindow = m_platformWindow->childWindowAt(windowPoint);
+    QPointF windowPoint;
+    QPointF screenPoint;
+    [self convertFromScreen:[NSEvent mouseLocation] toWindowPoint:&windowPoint andScreenPoint:&screenPoint];
+    QWindow *childWindow = m_platformWindow->childWindowAt(windowPoint.toPoint());
 
     // Top-level windows generate enter-leave events for sub-windows.
     // Qt wants to know which window (if any) will be entered at the
@@ -682,9 +703,9 @@ static QTouchDevice *touchDevice = 0;
     // handling mouseEnter and mouseLeave envents, since they are sent
     // individually to different views.
     if (m_platformWindow->m_nsWindow && childWindow) {
-        if (childWindow != m_platformWindow->m_underMouseWindow) {
-            QWindowSystemInterface::handleEnterLeaveEvent(childWindow, m_platformWindow->m_underMouseWindow, windowPoint, screenPoint);
-            m_platformWindow->m_underMouseWindow = childWindow;
+        if (childWindow != m_platformWindow->m_enterLeaveTargetWindow) {
+            QWindowSystemInterface::handleEnterLeaveEvent(childWindow, m_platformWindow->m_enterLeaveTargetWindow, windowPoint, screenPoint);
+            m_platformWindow->m_enterLeaveTargetWindow = childWindow;
         }
     }
 
@@ -698,6 +719,8 @@ static QTouchDevice *touchDevice = 0;
 
 - (void)mouseEntered:(NSEvent *)theEvent
 {
+    m_platformWindow->m_windowUnderMouse = true;
+
     if (m_window->flags() & Qt::WindowTransparentForInput)
         return [super mouseEntered:theEvent];
 
@@ -705,14 +728,17 @@ static QTouchDevice *touchDevice = 0;
     if (!m_platformWindow->m_nsWindow)
         return;
 
-    QPoint windowPoint, screenPoint;
-    [self convertFromEvent:theEvent toWindowPoint:&windowPoint andScreenPoint:&screenPoint];
-    m_platformWindow->m_underMouseWindow = m_platformWindow->childWindowAt(windowPoint);
-    QWindowSystemInterface::handleEnterEvent(m_platformWindow->m_underMouseWindow, windowPoint, screenPoint);
+    QPointF windowPoint;
+    QPointF screenPoint;
+    [self convertFromScreen:[NSEvent mouseLocation] toWindowPoint:&windowPoint andScreenPoint:&screenPoint];
+    m_platformWindow->m_enterLeaveTargetWindow = m_platformWindow->childWindowAt(windowPoint.toPoint());
+    QWindowSystemInterface::handleEnterEvent(m_platformWindow->m_enterLeaveTargetWindow, windowPoint, screenPoint);
 }
 
 - (void)mouseExited:(NSEvent *)theEvent
 {
+    m_platformWindow->m_windowUnderMouse = false;
+
     if (m_window->flags() & Qt::WindowTransparentForInput)
         return [super mouseExited:theEvent];
     Q_UNUSED(theEvent);
@@ -721,8 +747,8 @@ static QTouchDevice *touchDevice = 0;
     if (!m_platformWindow->m_nsWindow)
         return;
 
-    QWindowSystemInterface::handleLeaveEvent(m_platformWindow->m_underMouseWindow);
-    m_platformWindow->m_underMouseWindow = 0;
+    QWindowSystemInterface::handleLeaveEvent(m_platformWindow->m_enterLeaveTargetWindow);
+    m_platformWindow->m_enterLeaveTargetWindow = 0;
 }
 
 - (void)rightMouseDown:(NSEvent *)theEvent
@@ -775,6 +801,162 @@ static QTouchDevice *touchDevice = 0;
     [self handleMouseEvent:theEvent];
 }
 
+struct QCocoaTabletDeviceData
+{
+    QTabletEvent::TabletDevice device;
+    QTabletEvent::PointerType pointerType;
+    uint capabilityMask;
+    qint64 uid;
+};
+
+typedef QHash<uint, QCocoaTabletDeviceData> QCocoaTabletDeviceDataHash;
+Q_GLOBAL_STATIC(QCocoaTabletDeviceDataHash, tabletDeviceDataHash)
+
+- (void)handleTabletEvent: (NSEvent *)theEvent
+{
+    NSEventType eventType = [theEvent type];
+    if (eventType != NSTabletPoint && [theEvent subtype] != NSTabletPointEventSubtype)
+        return; // Not a tablet event.
+
+    ulong timestamp = [theEvent timestamp] * 1000;
+
+    QPointF windowPoint;
+    QPointF screenPoint;
+    [self convertFromScreen:[NSEvent mouseLocation] toWindowPoint: &windowPoint andScreenPoint: &screenPoint];
+
+    uint deviceId = [theEvent deviceID];
+    if (!tabletDeviceDataHash->contains(deviceId)) {
+        qWarning("QNSView handleTabletEvent: This tablet device is unknown"
+                 " (received no proximity event for it). Discarding event.");
+        return;
+    }
+    const QCocoaTabletDeviceData &deviceData = tabletDeviceDataHash->value(deviceId);
+
+    bool down = (eventType != NSMouseMoved);
+
+    qreal pressure;
+    if (down) {
+        pressure = [theEvent pressure];
+    } else {
+        pressure = 0.0;
+    }
+
+    NSPoint tilt = [theEvent tilt];
+    int xTilt = qRound(tilt.x * 60.0);
+    int yTilt = qRound(tilt.y * -60.0);
+    qreal tangentialPressure = 0;
+    qreal rotation = 0;
+    int z = 0;
+    if (deviceData.capabilityMask & 0x0200)
+        z = [theEvent absoluteZ];
+
+    if (deviceData.capabilityMask & 0x0800)
+        tangentialPressure = [theEvent tangentialPressure];
+
+    rotation = [theEvent rotation];
+
+    Qt::KeyboardModifiers keyboardModifiers = [QNSView convertKeyModifiers:[theEvent modifierFlags]];
+
+    QWindowSystemInterface::handleTabletEvent(m_window, timestamp, down, windowPoint, screenPoint,
+                                              deviceData.device, deviceData.pointerType, pressure, xTilt, yTilt,
+                                              tangentialPressure, rotation, z, deviceData.uid,
+                                              keyboardModifiers);
+}
+
+- (void)tabletPoint: (NSEvent *)theEvent
+{
+    if (m_window->flags() & Qt::WindowTransparentForInput)
+        return [super tabletPoint:theEvent];
+
+    [self handleTabletEvent: theEvent];
+}
+
+static QTabletEvent::TabletDevice wacomTabletDevice(NSEvent *theEvent)
+{
+    qint64 uid = [theEvent uniqueID];
+    uint bits = [theEvent vendorPointingDeviceType];
+    if (bits == 0 && uid != 0) {
+        // Fallback. It seems that the driver doesn't always include all the information.
+        // High-End Wacom devices store their "type" in the uper bits of the Unique ID.
+        // I'm not sure how to handle it for consumer devices, but I'll test that in a bit.
+        bits = uid >> 32;
+    }
+
+    QTabletEvent::TabletDevice device;
+    // Defined in the "EN0056-NxtGenImpGuideX"
+    // on Wacom's Developer Website (www.wacomeng.com)
+    if (((bits & 0x0006) == 0x0002) && ((bits & 0x0F06) != 0x0902)) {
+        device = QTabletEvent::Stylus;
+    } else {
+        switch (bits & 0x0F06) {
+            case 0x0802:
+                device = QTabletEvent::Stylus;
+                break;
+            case 0x0902:
+                device = QTabletEvent::Airbrush;
+                break;
+            case 0x0004:
+                device = QTabletEvent::FourDMouse;
+                break;
+            case 0x0006:
+                device = QTabletEvent::Puck;
+                break;
+            case 0x0804:
+                device = QTabletEvent::RotationStylus;
+                break;
+            default:
+                device = QTabletEvent::NoDevice;
+        }
+    }
+    return device;
+}
+
+- (void)tabletProximity: (NSEvent *)theEvent
+{
+    if (m_window->flags() & Qt::WindowTransparentForInput)
+        return [super tabletProximity:theEvent];
+
+    ulong timestamp = [theEvent timestamp] * 1000;
+
+    QCocoaTabletDeviceData deviceData;
+    deviceData.uid = [theEvent uniqueID];
+    deviceData.capabilityMask = [theEvent capabilityMask];
+
+    switch ([theEvent pointingDeviceType]) {
+        case NSUnknownPointingDevice:
+        default:
+            deviceData.pointerType = QTabletEvent::UnknownPointer;
+            break;
+        case NSPenPointingDevice:
+            deviceData.pointerType = QTabletEvent::Pen;
+            break;
+        case NSCursorPointingDevice:
+            deviceData.pointerType = QTabletEvent::Cursor;
+            break;
+        case NSEraserPointingDevice:
+            deviceData.pointerType = QTabletEvent::Eraser;
+            break;
+    }
+
+    deviceData.device = wacomTabletDevice(theEvent);
+
+    // The deviceID is "unique" while in the proximity, it's a key that we can use for
+    // linking up QCocoaTabletDeviceData to an event (especially if there are two devices in action).
+    bool entering = [theEvent isEnteringProximity];
+    uint deviceId = [theEvent deviceID];
+    if (entering) {
+        tabletDeviceDataHash->insert(deviceId, deviceData);
+    } else {
+        tabletDeviceDataHash->remove(deviceId);
+    }
+
+    if (entering) {
+        QWindowSystemInterface::handleTabletEnterProximityEvent(timestamp, deviceData.device, deviceData.pointerType, deviceData.uid);
+    } else {
+        QWindowSystemInterface::handleTabletLeaveProximityEvent(timestamp, deviceData.device, deviceData.pointerType, deviceData.uid);
+    }
+}
+
 - (void)touchesBeganWithEvent:(NSEvent *)event
 {
     const NSTimeInterval timestamp = [event timestamp];
@@ -803,6 +985,102 @@ static QTouchDevice *touchDevice = 0;
     QWindowSystemInterface::handleTouchEvent(m_window, timestamp * 1000, touchDevice, points);
 }
 
+#ifndef QT_NO_GESTURES
+//#define QT_COCOA_ENABLE_GESTURE_DEBUG
+- (void)magnifyWithEvent:(NSEvent *)event
+{
+#ifdef QT_COCOA_ENABLE_GESTURE_DEBUG
+    qDebug() << "magnifyWithEvent" << [event magnification];
+#endif
+    const NSTimeInterval timestamp = [event timestamp];
+    QPointF windowPoint;
+    QPointF screenPoint;
+    [self convertFromScreen:[NSEvent mouseLocation] toWindowPoint:&windowPoint andScreenPoint:&screenPoint];
+    QWindowSystemInterface::handleGestureEventWithRealValue(m_window, timestamp, Qt::ZoomNativeGesture,
+                                                            [event magnification], windowPoint, screenPoint);
+}
+
+#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_8
+- (void)smartMagnifyWithEvent:(NSEvent *)event
+{
+    static bool zoomIn = true;
+#ifdef QT_COCOA_ENABLE_GESTURE_DEBUG
+    qDebug() << "smartMagnifyWithEvent" << zoomIn;
+#endif
+    const NSTimeInterval timestamp = [event timestamp];
+    QPointF windowPoint;
+    QPointF screenPoint;
+    [self convertFromScreen:[NSEvent mouseLocation] toWindowPoint:&windowPoint andScreenPoint:&screenPoint];
+    QWindowSystemInterface::handleGestureEventWithRealValue(m_window, timestamp, Qt::SmartZoomNativeGesture,
+                                                            zoomIn ? 1.0f : 0.0f, windowPoint, screenPoint);
+    zoomIn = !zoomIn;
+}
+#endif
+
+- (void)rotateWithEvent:(NSEvent *)event
+{
+#ifdef QT_COCOA_ENABLE_GESTURE_DEBUG
+    qDebug() << "rotateWithEvent" << [event rotation];
+#endif
+    const NSTimeInterval timestamp = [event timestamp];
+    QPointF windowPoint;
+    QPointF screenPoint;
+    [self convertFromScreen:[NSEvent mouseLocation] toWindowPoint:&windowPoint andScreenPoint:&screenPoint];
+    QWindowSystemInterface::handleGestureEventWithRealValue(m_window, timestamp, Qt::RotateNativeGesture,
+                                                            -[event rotation], windowPoint, screenPoint);
+}
+
+- (void)swipeWithEvent:(NSEvent *)event
+{
+#ifdef QT_COCOA_ENABLE_GESTURE_DEBUG
+    qDebug() << "swipeWithEvent" << [event deltaX] << [event deltaY];
+#endif
+    const NSTimeInterval timestamp = [event timestamp];
+    QPointF windowPoint;
+    QPointF screenPoint;
+    [self convertFromScreen:[NSEvent mouseLocation] toWindowPoint:&windowPoint andScreenPoint:&screenPoint];
+
+    qreal angle = 0.0f;
+    if ([event deltaX] == 1)
+        angle = 180.0f;
+    else if ([event deltaX] == -1)
+        angle = 0.0f;
+    else if ([event deltaY] == 1)
+        angle = 90.0f;
+    else if ([event deltaY] == -1)
+        angle = 270.0f;
+
+    QWindowSystemInterface::handleGestureEventWithRealValue(m_window, timestamp, Qt::SwipeNativeGesture,
+                                                            angle, windowPoint, screenPoint);
+}
+
+- (void)beginGestureWithEvent:(NSEvent *)event
+{
+#ifdef QT_COCOA_ENABLE_GESTURE_DEBUG
+    qDebug() << "beginGestureWithEvent";
+#endif
+    const NSTimeInterval timestamp = [event timestamp];
+    QPointF windowPoint;
+    QPointF screenPoint;
+    [self convertFromScreen:[NSEvent mouseLocation] toWindowPoint:&windowPoint andScreenPoint:&screenPoint];
+    QWindowSystemInterface::handleGestureEvent(m_window, timestamp, Qt::BeginNativeGesture,
+                                               windowPoint, screenPoint);
+}
+
+- (void)endGestureWithEvent:(NSEvent *)event
+{
+#ifdef QT_COCOA_ENABLE_GESTURE_DEBUG
+    qDebug() << "endGestureWithEvent";
+#endif
+    const NSTimeInterval timestamp = [event timestamp];
+    QPointF windowPoint;
+    QPointF screenPoint;
+    [self convertFromScreen:[NSEvent mouseLocation] toWindowPoint:&windowPoint andScreenPoint:&screenPoint];
+    QWindowSystemInterface::handleGestureEvent(m_window, timestamp, Qt::EndNativeGesture,
+                                               windowPoint, screenPoint);
+}
+#endif // QT_NO_GESTURES
+
 #ifndef QT_NO_WHEELEVENT
 - (void)scrollWheel:(NSEvent *)theEvent
 {
@@ -855,28 +1133,46 @@ static QTouchDevice *touchDevice = 0;
     }
 #endif
 
-    QPoint qt_windowPoint, qt_screenPoint;
-    [self convertFromEvent:theEvent toWindowPoint:&qt_windowPoint andScreenPoint:&qt_screenPoint];
+    QPointF qt_windowPoint;
+    QPointF qt_screenPoint;
+    [self convertFromScreen:[NSEvent mouseLocation] toWindowPoint:&qt_windowPoint andScreenPoint:&qt_screenPoint];
     NSTimeInterval timestamp = [theEvent timestamp];
     ulong qt_timestamp = timestamp * 1000;
 
-    // Set keyboard modifiers depending on event phase. A two-finger trackpad flick
-    // generates a stream of scroll events. We want the keyboard modifier state to
-    // be the state at the beginning of the flick in order to avoid changing the
-    // interpretation of the events mid-stream. One example of this happening would
-    // be when pressing cmd after scrolling in Qt Creator: not taking the phase into
-    // account causes the end of the event stream to be interpreted as font size changes.
-
 #if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_7
     if ([theEvent respondsToSelector:@selector(scrollingDeltaX)]) {
-        NSEventPhase phase = [theEvent phase];
-        if (phase == NSEventPhaseBegan) {
+        // Prevent keyboard modifier state from changing during scroll event streams.
+        // A two-finger trackpad flick generates a stream of scroll events. We want
+        // the keyboard modifier state to be the state at the beginning of the
+        // flick in order to avoid changing the interpretation of the events
+        // mid-stream. One example of this happening would be when pressing cmd
+        // after scrolling in Qt Creator: not taking the phase into account causes
+        // the end of the event stream to be interpreted as font size changes.
+        NSEventPhase momentumPhase = [theEvent momentumPhase];
+        if (momentumPhase == NSEventPhaseNone) {
             currentWheelModifiers = [QNSView convertKeyModifiers:[theEvent modifierFlags]];
         }
 
-        QWindowSystemInterface::handleWheelEvent(m_window, qt_timestamp, qt_windowPoint, qt_screenPoint, pixelDelta, angleDelta, currentWheelModifiers);
-
+        NSEventPhase phase = [theEvent phase];
+        Qt::ScrollPhase ph = Qt::ScrollUpdate;
+#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_8
+        if (QSysInfo::QSysInfo::MacintoshVersion >= QSysInfo::MV_10_8) {
+            // On 10.8 and above, MayBegin is likely to happen.  We treat it the same as an actual begin.
+            if (phase == NSEventPhaseMayBegin)
+                ph = Qt::ScrollBegin;
+        } else
+#endif
+        if (phase == NSEventPhaseBegan) {
+            // On 10.7, MayBegin will not happen, so Began is the actual beginning.
+            ph = Qt::ScrollBegin;
+        }
         if (phase == NSEventPhaseEnded || phase == NSEventPhaseCancelled) {
+            ph = Qt::ScrollEnd;
+        }
+
+        QWindowSystemInterface::handleWheelEvent(m_window, qt_timestamp, qt_windowPoint, qt_screenPoint, pixelDelta, angleDelta, currentWheelModifiers, ph);
+
+        if (momentumPhase == NSEventPhaseEnded || momentumPhase == NSEventPhaseCancelled || momentumPhase == NSEventPhaseNone) {
             currentWheelModifiers = Qt::NoModifier;
         }
     } else
@@ -917,14 +1213,11 @@ static QTouchDevice *touchDevice = 0;
     NSString *charactersIgnoringModifiers = [nsevent charactersIgnoringModifiers];
     NSString *characters = [nsevent characters];
 
-    // [from Qt 4 impl] There is no way to get the scan code from carbon. But we cannot
+    // There is no way to get the scan code from carbon/cocoa. But we cannot
     // use the value 0, since it indicates that the event originates from somewhere
     // else than the keyboard.
     quint32 nativeScanCode = 1;
-
-    UInt32 nativeVirtualKey = 0;
-    EventRef eventRef = EventRef([nsevent eventRef]);
-    GetEventParameter(eventRef, kEventParamKeyCode, typeUInt32, 0, sizeof(nativeVirtualKey), 0, &nativeVirtualKey);
+    quint32 nativeVirtualKey = [nsevent keyCode];
 
     QChar ch = QChar::ReplacementCharacter;
     int keyCode = Qt::Key_unknown;
@@ -944,10 +1237,12 @@ static QTouchDevice *touchDevice = 0;
     if (ch.unicode() < 0xf700 || ch.unicode() > 0xf8ff)
         text = QCFString::toQString(characters);
 
+    QWindow *focusWindow = [self topLevelWindow];
+
     if (eventType == QEvent::KeyPress) {
 
         if (m_composingText.isEmpty())
-            m_sendKeyEvent = !QWindowSystemInterface::tryHandleShortcutEvent(m_window, timestamp, keyCode, modifiers, text);
+            m_sendKeyEvent = !QWindowSystemInterface::tryHandleShortcutEvent(focusWindow, timestamp, keyCode, modifiers, text);
 
         QObject *fo = QGuiApplication::focusObject();
         if (m_sendKeyEvent && fo) {
@@ -964,8 +1259,8 @@ static QTouchDevice *touchDevice = 0;
     }
 
     if (m_sendKeyEvent && m_composingText.isEmpty())
-        QWindowSystemInterface::handleExtendedKeyEvent(m_window, timestamp, QEvent::Type(eventType), keyCode, modifiers,
-                                                       nativeScanCode, nativeVirtualKey, nativeModifiers, text);
+        QWindowSystemInterface::handleExtendedKeyEvent(focusWindow, timestamp, QEvent::Type(eventType), keyCode, modifiers,
+                                                       nativeScanCode, nativeVirtualKey, nativeModifiers, text, [nsevent isARepeat]);
 
     m_sendKeyEvent = false;
 }
@@ -984,6 +1279,23 @@ static QTouchDevice *touchDevice = 0;
     [self handleKeyEvent:nsevent eventType:int(QEvent::KeyRelease)];
 }
 
+- (BOOL)performKeyEquivalent:(NSEvent *)nsevent
+{
+    NSString *chars = [nsevent charactersIgnoringModifiers];
+
+    if ([nsevent type] == NSKeyDown && [chars length] > 0) {
+        QChar ch = [chars characterAtIndex:0];
+        Qt::Key qtKey = qt_mac_cocoaKey2QtKey(ch);
+        // check for Command + Key_Period
+        if ([nsevent modifierFlags] & NSCommandKeyMask
+                && qtKey == Qt::Key_Period) {
+            [self handleKeyEvent:nsevent eventType:int(QEvent::KeyPress)];
+            return YES;
+        }
+    }
+    return [super performKeyEquivalent:nsevent];
+}
+
 - (void)flagsChanged:(NSEvent *)nsevent
 {
     ulong timestamp = [nsevent timestamp] * 1000;
@@ -1017,7 +1329,7 @@ static QTouchDevice *touchDevice = 0;
                                                timestamp,
                                                (lastKnownModifiers & mac_mask) ? QEvent::KeyRelease : QEvent::KeyPress,
                                                modifier_key_symbols[i].qt_code,
-                                               qmodifiers);
+                                               qmodifiers ^ [QNSView convertKeyModifiers:mac_mask]);
     }
 }
 
@@ -1276,7 +1588,7 @@ static QTouchDevice *touchDevice = 0;
                        NSFilenamesPboardType, NSPostScriptPboardType, NSTIFFPboardType,
                        NSRTFPboardType, NSTabularTextPboardType, NSFontPboardType,
                        NSRulerPboardType, NSFileContentsPboardType, NSColorPboardType,
-                       NSRTFDPboardType, NSHTMLPboardType, NSPICTPboardType,
+                       NSRTFDPboardType, NSHTMLPboardType,
                        NSURLPboardType, NSPDFPboardType, NSVCardPboardType,
                        NSFilesPromisePboardType, NSInkTextPboardType,
                        NSMultipleTextSelectionPboardType, mimeTypeGeneric, nil];
