@@ -49,6 +49,7 @@
 #include <qevent.h>
 #include <qfontmetrics.h>
 #include <qwindow.h>
+#include <qscreen.h>
 #include <qmainwindow.h>
 #include <qrubberband.h>
 #include <qstylepainter.h>
@@ -673,12 +674,18 @@ void QDockWidgetPrivate::updateButtons()
         = qobject_cast<QAbstractButton*>(dwLayout->widgetForRole(QDockWidgetLayout::FloatButton));
     button->setIcon(q->style()->standardIcon(QStyle::SP_TitleBarNormalButton, &opt, q));
     button->setVisible(canFloat && !hideButtons);
-
+#ifndef QT_NO_ACCESSIBILITY
+    button->setAccessibleName(QDockWidget::tr("Float"));
+    button->setAccessibleDescription(QDockWidget::tr("Undocks and re-attaches the dock widget"));
+#endif
     button
         = qobject_cast <QAbstractButton*>(dwLayout->widgetForRole(QDockWidgetLayout::CloseButton));
     button->setIcon(q->style()->standardIcon(QStyle::SP_TitleBarCloseButton, &opt, q));
     button->setVisible(canClose && !hideButtons);
-
+#ifndef QT_NO_ACCESSIBILITY
+    button->setAccessibleName(QDockWidget::tr("Close"));
+    button->setAccessibleDescription(QDockWidget::tr("Closes the dock widget"));
+#endif
     q->setAttribute(Qt::WA_ContentsPropagated,
                     (canFloat || canClose) && !hideButtons);
 
@@ -1246,7 +1253,7 @@ QDockWidget::DockWidgetFeatures QDockWidget::features() const
     window "on top" of its parent QMainWindow, instead of being
     docked in the QMainWindow.
 
-    By default, this property is true.
+    By default, this property is \c true.
 
     \sa isWindow()
 */
@@ -1259,6 +1266,9 @@ void QDockWidget::setFloating(bool floating)
         d->endDrag(true);
 
     QRect r = d->undockedGeometry;
+    // Keep position when undocking for the first time.
+    if (floating && isVisible() && !r.isValid())
+        r = QRect(mapToGlobal(QPoint(0, 0)), size());
 
     d->setWindowState(floating, false, floating ? r : QRect());
 
@@ -1297,8 +1307,8 @@ Qt::DockWidgetAreas QDockWidget::allowedAreas() const
 /*!
     \fn bool QDockWidget::isAreaAllowed(Qt::DockWidgetArea area) const
 
-    Returns true if this dock widget can be placed in the given \a area;
-    otherwise returns false.
+    Returns \c true if this dock widget can be placed in the given \a area;
+    otherwise returns \c false.
 */
 
 /*! \reimp */
@@ -1384,9 +1394,17 @@ bool QDockWidget::event(QEvent *event)
         d->toggleViewAction->setChecked(false);
         emit visibilityChanged(false);
         break;
-    case QEvent::Show:
+    case QEvent::Show: {
         d->toggleViewAction->setChecked(true);
-        emit visibilityChanged(geometry().right() >= 0 && geometry().bottom() >= 0);
+        QPoint parentTopLeft(0, 0);
+        if (isWindow()) {
+            if (const QWindow *window = windowHandle())
+                parentTopLeft = window->screen()->availableVirtualGeometry().topLeft();
+            else
+                parentTopLeft = QGuiApplication::primaryScreen()->availableVirtualGeometry().topLeft();
+        }
+        emit visibilityChanged(geometry().right() >= parentTopLeft.x() && geometry().bottom() >= parentTopLeft.y());
+}
         break;
 #endif
     case QEvent::ApplicationLayoutDirectionChange:
